package com.hungraim.ltc.account.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.account.service.EmpAccountService;
import com.hungraim.ltc.pojo.entity.account.ChDisabEmpAccounts;
import com.hungraim.ltc.pojo.vo.account.EmpAccountOperVo;
import com.hungraim.ltc.pojo.vo.account.SrvEmpListReq;
import com.hungraim.ltc.pojo.vo.account.SrvEmpListResp;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;

/**
 * 结算管理控制层
 */
@RestController
@RequestMapping("/api.account/empAccount")
@AllArgsConstructor
@Slf4j
public class EmpAccountController {

    private final EmpAccountService empAccountService;

    /**
     * 添加或修改结算记录
     *
     * @return 返回状态
     */
    @PostMapping("/addOrUpdateEmpAccount")
    public Result addOrUpdateEmpAccount(EmpAccountOperVo empAccountOperVo) {
        if (null == empAccountOperVo.getSrvOrganId() || null == empAccountOperVo.getAccountsTime() || StringUtils.isBlank(empAccountOperVo.getTotalExpensesPayable()) || null == empAccountOperVo.getAccountsStatus()) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        Long empAccountsId = empAccountOperVo.getEmpAccountsId();
        Date accountsTime = empAccountOperVo.getAccountsTime();
        if (null == empAccountsId) {
            List<ChDisabEmpAccounts> list = empAccountService.list(new LambdaQueryWrapper<ChDisabEmpAccounts>().eq(ChDisabEmpAccounts::getAccountsTime, accountsTime).eq(ChDisabEmpAccounts::getSrvEmpId, empAccountOperVo.getSrvEmpId()));
            if (null != list && list.size() > 0) {
                return Result.failed("当前人员当月已存在结算数据");
            }
        }
        empAccountService.addOrUpdateEmpAccount(empAccountOperVo);
        return Result.success();
    }

    /**
     * 查询结算信息列表
     *
     * @param srvEmpListReq 请求参数对象
     * @return 返回列表
     */
    @GetMapping("/searchSrvEmpList")
    public Result<Page<SrvEmpListResp>> searchSrvEmpList(SrvEmpListReq srvEmpListReq) {
        return Result.success(empAccountService.searchSrvEmpList(srvEmpListReq));
    }

    /**
     * 获取结算详情
     *
     * @param empAccountsId 结算id
     * @return 返回机构结算详情
     */
    @GetMapping("/srvEmpDetail")
    public Result<EmpAccountOperVo> getSrvEmpDetail(Long empAccountsId) {
        if (null == empAccountsId) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        return Result.success(empAccountService.getSrvEmpDetail(empAccountsId));
    }
}
