package com.hungraim.ltc.account.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hungraim.ltc.account.dao.ChDisabEmpAccountsMapper;
import com.hungraim.ltc.account.service.BaseAccountService;
import com.hungraim.ltc.account.service.EmpAccountService;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.account.ChDisabEmpAccounts;
import com.hungraim.ltc.pojo.vo.account.EmpAccountOperVo;
import com.hungraim.ltc.pojo.vo.account.SrvEmpListReq;
import com.hungraim.ltc.pojo.vo.account.SrvEmpListResp;
import com.hungraim.ltc.util.AttachType;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@AllArgsConstructor
@Slf4j
public class EmpAccountServiceImpl extends ServiceImpl<ChDisabEmpAccountsMapper, ChDisabEmpAccounts> implements EmpAccountService {

    private final AttachFeignService attachFeignService;
    private final ChDisabEmpAccountsMapper chDisabEmpAccountsMapper;
    private final BaseAccountService baseAccountService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addOrUpdateEmpAccount(EmpAccountOperVo empAccountOperVo) {
        ChDisabEmpAccounts chDisabEmpAccounts = new ChDisabEmpAccounts();
        BeanUtils.copyProperties(empAccountOperVo, chDisabEmpAccounts);
        chDisabEmpAccounts.setChooseSettle((short) 0); // 是否结算(0未结算，1已结算)
        this.saveOrUpdate(chDisabEmpAccounts);
        empAccountOperVo.setEmpAccountsId(chDisabEmpAccounts.getEmpAccountsId());
        baseAccountService.updateAttachs(chDisabEmpAccounts.getEmpAccountsId(), empAccountOperVo.getFileDataList(), AttachType.SRV_EMP_ACCOUNT);

    }

    @Override
    public Page<SrvEmpListResp> searchSrvEmpList(SrvEmpListReq srvEmpListReq) {
        List<Long> organIds = baseAccountService.getOrganIds(srvEmpListReq.getOrganId(), srvEmpListReq.getIsInvolveChild());
        return chDisabEmpAccountsMapper.searchSrvEmpList(new Page<>(srvEmpListReq.getPageNum(), srvEmpListReq.getPageSize()), organIds, srvEmpListReq);
    }


    @Override
    public EmpAccountOperVo getSrvEmpDetail(Long empAccountsId) {
        EmpAccountOperVo empAccountOperVo = chDisabEmpAccountsMapper.getDetailById(empAccountsId);
        try {
            Result<List<ChFndAttach>> result = attachFeignService.selectAttachs(empAccountOperVo.getEmpAccountsId(), AttachType.SRV_ORGAN_ACCOUNT.getUploadType());
            if (ResultCode.SUCCESS.getCode().equals(result.getCode())) {
                empAccountOperVo.setFileDataList(result.getData());
            } else {
                log.error("获取机构结算资料失败：{}", result.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取机构结算资料失败：{}", e.getMessage());
        }
        return empAccountOperVo;
    }
}
