package com.hungraim.ltc.controller;

import com.hungraim.ltc.service.OrganService;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Slf4j
@RestController
@RequestMapping("/api.apply/organ")
@AllArgsConstructor
public class OrganController {

    private final OrganService organService;

    /**
     * 根据organId以及isInvolveChild返回一个organIds集合
     * 返回一个好办organId的ids集合，如果isInvolveChild的值为true，则包含organId的下层机构，如果为false，就只返回只存在organId的ids集合
     *
     * @param organId        适用机构id
     * @param isInvolveChild 是否包含下层机构
     * @return 返回一个ids集合
     */
    @GetMapping("/child")
    public Result<List<Long>> getOrganChild(@RequestParam("organId") Long organId, @RequestParam("isInvolveChild") Boolean isInvolveChild){
        if (null == organId || null == isInvolveChild) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        log.info("getOrganChild:{},{}",organId,isInvolveChild);
        return Result.success(organService.getOrganIds(organId, isInvolveChild));
    }
}
