package com.hungraim.ltc.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskDetail;
import com.hungraim.ltc.pojo.vo.disable.ChSrvTaskDetailVo;
import com.hungraim.ltc.pojo.vo.disable.SrvOrganEmpListVo;
import com.hungraim.ltc.pojo.vo.disable.SrvOrganEmpVo;
import org.apache.poi.ss.usermodel.Workbook;

import java.util.List;

public interface SrvOrganEmpService {

    /**
     * 分页查询护理人员列表
     *
     * @param page           页码参数
     * @param organId        适用机构id
     * @param isInvolveChild 是否包含子机构
     * @param empName        人员姓名
     * @param srvOrganName   护理机构名称
     * @param certifNum      身份证号
     * @param isLock         人员状态
     * @param empType        护理人员类型
     * @return 返回列表
     */
    Page<SrvOrganEmpListVo> getSrvOrganEmpListVo(Page<SrvOrganEmpListVo> page, Long organId, Boolean isInvolveChild, String empName, String srvOrganName, String certifNum, Short isLock, Short empType);

    /**
     * 修改护理人员的状态
     *
     * @param empId  护理人员id
     * @param isLock 状态  0-有效；1-失效；
     * @return 返回操作状态
     */
    void updateSrvEmpStatus(Long empId, Short isLock);

    /**
     * 获取护理人员相关信息
     *
     * @param empId 护理人员id
     * @return 返回护理人员信息
     */
    SrvOrganEmpVo getSrvOrganEmp(Long empId);

    /**
     * 保存或修改护理人员对象
     *
     * @param srvOrganEmpVo 护理人员对象参数
     *                      empId:"", // 护理人员id
     *                      organId: "", // 适用机构id
     *                      srvOrganId: "", // 护理机构id
     *                      unifiedSocialCreditCode: "", // 统一社会信用代码
     *                      empName: "", // 护理人员姓名
     *                      certifType: "", // 护理人员证件类型
     *                      certifNum: "", // 护理人员证件号码
     *                      gender: "", // 护理人员性别
     *                      age: "", // 护理人员年龄
     *                      mobilePhone: "", // 护理人员手机号
     *                      empTypeId: "", // 护理人员资质
     *                      srvCateId: "", // 护理人员类别
     *                      empType: '', // 人员类型
     *                      eduId: "", // 文化水平
     *                      fileDataList: [], // 对应数据库中的文件数据列表
     */
    void saveOrUpdateSrvOrganEmp(SrvOrganEmpVo srvOrganEmpVo);

    Workbook srvOrganExport(Long organId, Boolean isInvolveChild, String empName, String srvOrganName, String certifNum,
                            Short isLock, Short empType);

    Page<ChSrvTaskDetailVo> chSrvTaskDetailQuery(Page<ChSrvTaskDetailVo> page,Long empId);

    int chSrvTaskDetailDelete(long taskDetailId,String userName);
}
