package com.hungraim.ltc.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.dao.*;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.disable.ChCfgSrvEmpType;
import com.hungraim.ltc.pojo.entity.disable.ChSrvOrgan;
import com.hungraim.ltc.pojo.entity.riskLevel.ChSrvempRiskLevel;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvOrganEmp;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskDetail;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskDetailDelete;
import com.hungraim.ltc.pojo.vo.disable.ChSrvTaskDetailVo;
import com.hungraim.ltc.pojo.vo.disable.SrvOrganEmpExportVo;
import com.hungraim.ltc.pojo.vo.disable.SrvOrganEmpListVo;
import com.hungraim.ltc.pojo.vo.disable.SrvOrganEmpVo;
import com.hungraim.ltc.pojo.vo.system.AttachUpdateVo;
import com.hungraim.ltc.service.OrganService;
import com.hungraim.ltc.service.SrvOrganEmpService;
import com.hungraim.ltc.util.AttachType;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@Service
@AllArgsConstructor
@Slf4j
public class SrvOrganEmpServiceImpl implements SrvOrganEmpService {

    private final ChSrvOrganEmpMapper chSrvOrganEmpMapper;

    private final OrganService organService;

    private final AttachFeignService attachFeignService;

    private final SrvEmpTypeMapper srvEmpTypeMapper;

    private final SrvOrganMapper srvOrganMapper;

    @Autowired
    private ChSrvempRiskLevelMapper chSrvempRiskLevelMapper;

    private final ChSrvTaskDetailMapper chSrvTaskDetailMapper;

    private final ChSrvTaskDetailDeleteMapper chSrvTaskDetailDeleteMapper;



    @Override
    public Page<SrvOrganEmpListVo> getSrvOrganEmpListVo(Page<SrvOrganEmpListVo> page, Long organId, Boolean isInvolveChild, String empName, String srvOrganName, String certifNum, Short isLock, Short empType) {
        List<Long> organIds = organService.getOrganIds(organId, isInvolveChild);
        return chSrvOrganEmpMapper.getSrvOrganEmpListVo(page, organIds, empName, srvOrganName, certifNum, isLock, empType);
    }

    @Override
    public void updateSrvEmpStatus(Long empId, Short isLock) {
        ChSrvOrganEmp chSrvOrganEmp = new ChSrvOrganEmp();
        chSrvOrganEmp.setEmpId(empId);
        chSrvOrganEmp.setIsLock(isLock);
        chSrvOrganEmpMapper.updateById(chSrvOrganEmp);
    }

    @Override
    public SrvOrganEmpVo getSrvOrganEmp(Long empId) {
        SrvOrganEmpVo srvOrganEmpVo = new SrvOrganEmpVo();
        ChSrvOrganEmp chSrvOrganEmp = chSrvOrganEmpMapper.selectById(empId);
        Long empTypeId = chSrvOrganEmp.getEmpTypeId();
        if(empTypeId!=null){
            ChCfgSrvEmpType chCfgSrvEmpType = srvEmpTypeMapper.selectById(empTypeId);
            srvOrganEmpVo.setEmpCateId(chCfgSrvEmpType.getEmpCateId());
        }else{
            srvOrganEmpVo.setEmpCateId(0);
        }
        Long srvOrganId = chSrvOrganEmp.getSrvOrganId();
        ChSrvOrgan chSrvOrgan = srvOrganMapper.selectById(srvOrganId);
        BeanUtils.copyProperties(chSrvOrganEmp, srvOrganEmpVo);
        srvOrganEmpVo.setUnifiedSocialCreditCode(chSrvOrgan.getUnifiedSocialCreditCode());
        try {
            Result<List<ChFndAttach>> listResult = attachFeignService.selectAttachs(empId, AttachType.SRV_ORGAN_EMP.getUploadType());
            if (ResultCode.SUCCESS.getCode().equals(listResult.getCode())) {
                srvOrganEmpVo.setFileDataList(listResult.getData());
            } else {
                log.error("请求获取护理人员上传文件出错，出错信息：{}", listResult.getMsg());
            }
        } catch (Exception e) {
            log.error("请求获取护理人员上传文件出错，出错信息：{}", e.getMessage());
        }
        return srvOrganEmpVo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveOrUpdateSrvOrganEmp(SrvOrganEmpVo srvOrganEmpVo) {
        Long empId = srvOrganEmpVo.getEmpId();
        ChSrvOrganEmp chSrvOrganEmp = new ChSrvOrganEmp();
        BeanUtils.copyProperties(srvOrganEmpVo, chSrvOrganEmp);
        if (null == empId) {
            // 新增
            chSrvOrganEmp.setEffTime(new Date());
            chSrvOrganEmp.setIsLock((short) 0);
            chSrvOrganEmpMapper.insert(chSrvOrganEmp);
            updateAttachs(srvOrganEmpVo, chSrvOrganEmp.getEmpId());

            ChSrvempRiskLevel chSrvempRiskLevel = new ChSrvempRiskLevel();
            chSrvempRiskLevel.setRiskLevel(0L);
            chSrvempRiskLevel.setSrvEmpId(Long.valueOf(chSrvOrganEmp.getEmpId()));
            chSrvempRiskLevel.setStatus(0);
            chSrvempRiskLevel.setFcd(new Date());
            chSrvempRiskLevelMapper.insert(chSrvempRiskLevel);

        } else {
            // 修改
            chSrvOrganEmpMapper.updateById(chSrvOrganEmp);
            updateAttachs(srvOrganEmpVo, empId);
        }
    }

    // 抽取方法
    private void updateAttachs(SrvOrganEmpVo srvOrganEmpVo, Long empId) {
        AttachUpdateVo attachUpdateVo = new AttachUpdateVo();
        attachUpdateVo.setChFndAttachs(srvOrganEmpVo.getFileDataList());
        attachUpdateVo.setCtrlId(empId);
        attachUpdateVo.setAttachType(AttachType.SRV_ORGAN_EMP.getUploadType());
        attachFeignService.updateAttachs(attachUpdateVo);
    }


    @Override
    public Workbook srvOrganExport(Long organId, Boolean isInvolveChild, String empName, String srvOrganName, String certifNum,
                                   Short isLock, Short empType) {
        List<Long> organIds = organService.getOrganIds(organId, isInvolveChild);
        List<SrvOrganEmpExportVo> srvOrganEmpExportList = chSrvOrganEmpMapper.getSrvOrganEmpExportList(organIds, empName, srvOrganName, certifNum, isLock, empType);

        for (SrvOrganEmpExportVo record : srvOrganEmpExportList) {
            StringBuilder stringBuilder = new StringBuilder();
            if(StringUtils.isEmpty(record.getProvincial())){
                stringBuilder.append("--");
            }else if( record.getProvincial().equals(record.getCity())){
                stringBuilder.append(record.getProvincial());
            }else {
                stringBuilder.append(record.getProvincial()).append(record.getCity());
            }
            record.setProvinCialcityArea(stringBuilder.toString());
        }
        ExportParams exportParams = new ExportParams();
        exportParams.setType(ExcelType.XSSF); // 对应xlsx
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, SrvOrganEmpExportVo.class, srvOrganEmpExportList);

        return workbook;
    }

    @Override
    public Page<ChSrvTaskDetailVo> chSrvTaskDetailQuery(Page<ChSrvTaskDetailVo> page,Long empId) {
        return chSrvTaskDetailMapper.chSrvTaskDetailQuery(page,empId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int chSrvTaskDetailDelete(long taskDetailId,String userName) {
        ChSrvTaskDetail chSrvTaskDetail = chSrvTaskDetailMapper.selectById(taskDetailId);
        ChSrvTaskDetailDelete chSrvTaskDetailDelete = new ChSrvTaskDetailDelete();
        BeanUtils.copyProperties(chSrvTaskDetail,chSrvTaskDetailDelete);
        chSrvTaskDetailDelete.setDeleteDate(new Date());
        chSrvTaskDetailDelete.setDeleteName(userName);
        chSrvTaskDetailDeleteMapper.insert(chSrvTaskDetailDelete);
        return chSrvTaskDetailMapper.deleteById(taskDetailId);
    }

}
