package com.hungraim.ltc.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

/**
 * 文件工具类
 */
@Slf4j
public class FileUtils {

    /**
     * 上传文件
     *
     * @param uploadFile controller接收到的文件对象
     * @param path       存放文件的根路径
     * @return 返回文件具体保存的路径，包含文件名称
     */
    public static String uploadFile(MultipartFile uploadFile, String path) {
        log.info("开始进行文件上传");
        try {
            //获取到文件名
            String fileName = uploadFile.getOriginalFilename();
            log.info("文件名称：{}", fileName);
            //使用日期格式创建文件夹
            String dataPathDir = new SimpleDateFormat("yyyy/MM/dd").format(new Date());
            //真实的文件夹路径
            String realDirPath = path + "/" + dataPathDir;
            //判断文件夹是否存在
            File dirFile = new File(realDirPath);
            if (!dirFile.exists()) {
                dirFile.mkdirs();
            }
            String uuid = UUID.randomUUID().toString().replace("-", "");
            //截取原先文件的类型
            String fileType = fileName.substring(fileName.lastIndexOf("."));
            String realName = uuid + fileType;
            //文件上传
            String realFilePath = realDirPath + "/" + realName;
            log.info("文件上传路径以及名称：{}", realFilePath);
            File file = new File(realFilePath);
            uploadFile.transferTo(file);
            return realFilePath;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     *@author zwl
     *@desc 导出返回设置
     *@date 2023/1/12
     *@param  * @param workbook
     * @param fileName
     * @param response
     *@return void
     **/
    public static void  exportResponse(Workbook workbook,String fileName, HttpServletResponse response){
        try{
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition","attachment;filename=" + URLEncoder.encode(fileName,"UTF-8"));
            // 输出流
            workbook.write(response.getOutputStream());
            // 清除response
            response.flushBuffer();
        }catch (Exception e){
            log.error(e.getMessage());
        }
    }

}
