package com.hungraim.ltc.util;

import com.alibaba.fastjson.JSONObject;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.*;
import java.net.Socket;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class SocketCallUtil {

    private final static Logger logger = LogManager.getLogger(SocketCallUtil.class);
    /**
     * 调用前置机
     * @param ip
     * @param port
     * @param jsonStr 需要传输给前置机的报文
     * @return
     * @throws Exception
     */
    public static String callSocket(String ip, int port, String jsonStr) {
        Socket socket = null;
        InputStream in = null;
        PrintWriter out = null;
        try {
            socket = new Socket(ip, port);
            out = new PrintWriter(new OutputStreamWriter(socket.getOutputStream(), "utf-8"));
            jsonStr = URLEncoder.encode(jsonStr, "UTF-8");
            String oraMessage = "http://iai.tencentcloudapi.com/Method=TCY&outputStr=" + jsonStr;
            String length8Str = getByteLength8(oraMessage);
            String message = "readFully" + length8Str + "nodecrypt" + oraMessage;
            out.print(message);
            out.flush();
            in = socket.getInputStream();
            Thread.sleep(1000);
            byte[] buf = new byte[1024];
            List<Byte> byteList = new ArrayList<Byte>();
            while ((in.read(buf)) != -1) {
                for (byte bt : buf) {
                    byteList.add(bt);
                }
            }
            byte[] b = new byte[byteList.size()];
            int i = 0;
            Iterator<Byte> iterator = byteList.iterator();
            while (iterator.hasNext()) {
                b[i] = iterator.next();
                i++;
            }
            byte[] byteLength8 = new byte[8];
            System.arraycopy(b, 0, byteLength8, 0, 8);
            int length8 = Integer.parseInt(new String(byteLength8));
            byte[] bytes = new byte[length8];
            System.arraycopy(b, 8, bytes, 0, length8);
            return URLDecoder.decode(new String(bytes, "UTF-8"), "UTF-8");
        } catch (Exception e) {
            e.printStackTrace();
            return "调用前置机异常";
        } finally {
            try {
                in.close();
                out.close();
                if (!socket.isClosed()) {
                    socket.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

    }

    /**
     * 获取byte字符的8位长度（不足八位左补零） add by oydg 20140902
     *
     * @param content
     * @return
     */
    private static String getByteLength8(String content) {
        String length8 = "";
        if (null == content) {
            return length8;
        }
        String length = (content.length() + 8) + "";
        int size = length.length();
        for (int i = 0; i < 8 - size; i++) {
            length = "0" + length;
        }
        length8 = length;
        return length8;
    }

    /**
     * 通过socket连接前置机进行互联网接口的对接
     *
     * @param socketIp          前置机IP
     * @param socketPort        前置机端口
     * @param url               目标请求的URL
     * @param characterEncoding 字符集编码
     * @param isProxy           是否开启代理请求
     * @return
     */
    public static String commentSocketUtil(String socketIp, int socketPort, String url, String input, String characterEncoding, String isProxy) {
        Socket socket = null;
        try {
            socket = new Socket(socketIp, socketPort);
            JSONObject json = new JSONObject();
            json.put("openProxy", isProxy);//必传 启用代理 TRUE、FALSE，一般传TRUE，不要传FALSE
            json.put("encode", characterEncoding);//非必传，不传默认UTF-8，如http(s)接口提供方反馈中文乱码，按对方编码设置
            json.put("contentType", "application/json");//非必传 Content-Type 按实际需要填写
            if("deleteVideoFile".equals(input)){
                json.put("requestMethod", "GET");//必传 POST、GET
            }else{
                json.put("requestMethod", "POST");//必传 POST、GET
                json.put("outputString",input);//参数
            }
            String jsonStr = URLEncoder.encode(json.toString(), characterEncoding);
            String oraMessage = url + "Method=WAN&outputStr=" + jsonStr;
            String length8Str = getByteLength8(oraMessage);
            logger.info("开始连接前置机,请求的目标地址为:" + url);
            if (socket.isConnected()) {
                InputStream in = new BufferedInputStream(socket.getInputStream());
                OutputStream out = socket.getOutputStream();
                String message = "readFully" + length8Str + "nodecrypt" + oraMessage;
                out.write((message).getBytes());
                out.flush();
                byte htmlByte[] = readInputStream(in);
                String result = new String(htmlByte, characterEncoding);
                logger.info("前置机返回外部接口信息:" + url);
                return result;
            } else {
                logger.info("程序正常，但是连接前置机失败!");
                return "程序正常，但是连接前置机失败!";
            }
        } catch (Exception e) {
            logger.info("前置机socket访问外部接口连接失败：" + e.getStackTrace());
            return "查询失败";
        } finally {
            if (!socket.isClosed()) {
                try {
                    socket.close();
                } catch (Exception e) {
                    logger.info("关闭前置机socket连接错误：" + e.getStackTrace());
                }
            }

        }
    }

    /**
     * 流转byte[]，无需修改
     *
     * @param inStream
     * @return
     * @throws Exception
     */
    private static byte[] readInputStream(InputStream inStream) throws Exception {
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        byte[] buffer = new byte[9024];
        int len = 0;
        while ((len = inStream.read(buffer)) != -1) {
            outStream.write(buffer, 0, len);
        }
        inStream.close();
        return outStream.toByteArray();
    }

}
