package com.hungraim.ltc.governance.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.governance.service.AprOrganService;
import com.hungraim.ltc.pojo.entity.assessment.ChCfgAprGroupDetail;
import com.hungraim.ltc.pojo.entity.serviceManage.ChAprOrgan;
import com.hungraim.ltc.pojo.entity.serviceManage.ChCfgAprOrganType;
import com.hungraim.ltc.pojo.entity.serviceManage.ChCfgAprempType;
import com.hungraim.ltc.pojo.entity.serviceManage.ChFndEducation;
import com.hungraim.ltc.pojo.vo.governance.AprEmpInfoVO;
import com.hungraim.ltc.pojo.vo.governance.AprEmpVO;
import com.hungraim.ltc.pojo.vo.governance.AprOrganInfoVO;
import com.hungraim.ltc.pojo.vo.governance.AprOrganVO;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 评定机构及评定机构人员信息管理Controller
 *
 * @author czz
 */
@RestController
@RequestMapping("/api.governance/organ")
@Slf4j
public class AprOrganController {

    @Autowired
    private AprOrganService aprOrganService;

    /**
     * 评定机构信息列表
     *
     * @param pageNum    页码
     * @param pageSize   每页展示条数
     * @param organId    适用机构id
     * @param isIncluded 是否包含下级机构,值为true，则包含organId的下层机构
     * @param organName  机构名称
     * @param code       统一社会信用代码
     */
    @RequestMapping("/aprOrganList")
    public Result<Page<AprOrganVO>> aprOrganList(@RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "10") Integer pageSize,
                                                 Long organId, Boolean isIncluded, String organName, String code) {
        Page<AprOrganVO> page = new Page<>(pageNum, pageSize);
        page = aprOrganService.aprOrganList(page, organId, isIncluded, organName, code);
        return Result.success(page);
    }

    /**
     * 通过organId获取aprOrgan集合
     *
     * @param organId 适用机构id
     * @return 返回评定机构列表
     */
    @GetMapping("/aprOrganListByOrganId")
    public Result<List<ChAprOrgan>> getAprOrganListByOrganId(Long organId) {
        if (null == organId) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        String status = "0";//有效
        List<ChAprOrgan> list = aprOrganService.list(new LambdaQueryWrapper<ChAprOrgan>().eq(ChAprOrgan::getOrganId, organId).eq(ChAprOrgan::getStatus, status));
        return Result.success(list);
    }

    /**
     * 获取评定机构信息
     *
     * @param aprOrganId 评定机构id
     * @return 返回相关信息
     */
    @GetMapping("/getAprOrganInfo")
    public Result<AprOrganInfoVO> getAprOrganInfo(Long aprOrganId) {
        if (null == aprOrganId) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        AprOrganInfoVO aprOrganInfoVO = aprOrganService.getAprOrganInfo(aprOrganId);
        return Result.success(aprOrganInfoVO);
    }

    /**
     * 更改评定机构状态
     *
     * @param aprOrganId 监管机构id
     * @param status     状态 有效：0；失效：1
     */
    @DeleteMapping("/updateAprOrganStatus")
    public Result updateAprOrganStatus(String aprOrganId, Short status) {
        if (StringUtils.isBlank(aprOrganId) || status == null || (status != 1 && status != 0)) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        aprOrganService.updateAprOrganStatus(aprOrganId, status);
        return Result.success();
    }

    /**
     * 新增评定机构或更新评定机构信息(有apr_organ_id就更新,没有就新增)
     */
    @PostMapping("/addOrUpdateAprOrgan")
    public Result addOrUpdateAprOrgan(AprOrganInfoVO chAprOrgan) {
        if (chAprOrgan == null) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (chAprOrgan.getFileDataList().size() < 3) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (chAprOrgan.getAprOrganId() == null) {
            //新增
            if (StringUtils.isBlank(chAprOrgan.getOrganId())) {//适用机构
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getDistrictProvincial()) || StringUtils.isBlank(chAprOrgan.getDistrictArea()) || StringUtils.isBlank(chAprOrgan.getDistrictCity())) {//所在省市区
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getOrganName())) {//评定机构名称
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getUnifiedSocialCreditCode())) {//统一社会信用代码
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (chAprOrgan.getOrganTypeId() == null) {//机构类型id
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getAcptProvincial()) || StringUtils.isBlank(chAprOrgan.getAcptCity()) || StringUtils.isBlank(chAprOrgan.getAcptArea()) || StringUtils.isBlank(chAprOrgan.getAcptAddress()) || StringUtils.isBlank(chAprOrgan.getAcptStreet())) {//机构详细地址
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getLinkName())) {//机构联系人姓名
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getLinkTel())) {//机构联系人手机号
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (chAprOrgan.getGroupDetailName() == null) {//评定起点地址
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getAccName())) {//银行账户名称
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getBankNo())) {//银行账号
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getBankDistrictProvincial()) || StringUtils.isBlank(chAprOrgan.getBankDistrictCity())) {//账号所属省市
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getBankDistrictWebsite())) {//账号所属网点
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (chAprOrgan.getCardType() == null) {//卡类型
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
        } else {
            //更新
            if (StringUtils.isBlank(chAprOrgan.getAccName())) {//银行账户名称
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getBankNo())) {//银行账号
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getBankDistrictProvincial()) || StringUtils.isBlank(chAprOrgan.getBankDistrictCity())) {//账号所属省市
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getBankDistrictWebsite())) {//账号所属网点
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (chAprOrgan.getCardType() == null) {//卡类型
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getDistrictProvincial()) || StringUtils.isBlank(chAprOrgan.getDistrictArea()) || StringUtils.isBlank(chAprOrgan.getDistrictCity())) {//所在省市区
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getUnifiedSocialCreditCode())) {//统一社会信用代码
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (chAprOrgan.getOrganTypeId() == null) {//机构类型id
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getAcptArea()) || StringUtils.isBlank(chAprOrgan.getAcptAddress()) || StringUtils.isBlank(chAprOrgan.getAcptStreet())) {//机构详细地址
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getLinkName())) {//机构联系人姓名
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (StringUtils.isBlank(chAprOrgan.getLinkTel())) {//机构联系人手机号
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
            if (chAprOrgan.getGroupDetailName() == null) {//评定起点地址
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            }
        }
        aprOrganService.addOrUpdateAprOrgan(chAprOrgan);
        return Result.success();
    }

    /**
     * 获取所有鉴定机构类型
     */
    @GetMapping("/cfgAprOrganTypes")
    public Result<List<ChCfgAprOrganType>> getCfgAprOrganTypes() {
        List<ChCfgAprOrganType> chCfgAprOrganTypes = aprOrganService.getCfgAprOrganTypes();
        return Result.success(chCfgAprOrganTypes);
    }

    /**
     * 获取评定参数列表
     */
    @GetMapping("/cfgAprGroupDetail")
    public Result<List<ChCfgAprGroupDetail>> getCfgAprGroupDetail() {
        List<ChCfgAprGroupDetail> chCfgAprGroupDetails = aprOrganService.getCfgAprGroupDetail();
        return Result.success(chCfgAprGroupDetails);
    }

    /**
     * 评定人员信息列表
     *
     * @param pageNum    页码
     * @param pageSize   每页展示条数
     * @param organId    适用机构id
     * @param isIncluded 是否包含下级机构,值为true，则包含organId的下层机构
     * @param organName  评定机构名称
     * @param realName   评定人员姓名
     * @param tel        评定人员手机号码
     * @param certiCode  评定人员证件号码
     * @param isLock     状态 有效：0；失效：1
     * @param empType    评定人员类型 鉴定人员：0；管理人员：1
     */
    @RequestMapping("/aprEmpList")
    public Result<Page<AprEmpVO>> aprEmpList(@RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "10") Integer pageSize,
                                             Long organId, Boolean isIncluded, String organName, String realName, String tel, String certiCode, String isLock, String empType) {
        Page<AprEmpVO> page = new Page<>(pageNum, pageSize);
        page = aprOrganService.aprEmpList(page, organId, isIncluded, organName, realName, tel, certiCode, isLock, empType);
        return Result.success(page);
    }

    /**
     * 获取评定机构人员信息
     *
     * @param empId 评定机构人员id
     * @return 返回相关信息
     */
    @GetMapping("/getAprEmpInfo")
    public Result<AprEmpInfoVO> getAprEmpInfo(Long empId) {
        if (null == empId) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        AprEmpInfoVO aprEmpInfoVO = aprOrganService.getAprEmpInfo(empId);
        return Result.success(aprEmpInfoVO);
    }

    /**
     * 新增评定机构人员或更新评定机构人员信息(有emp_id就更新,没有就新增)
     */
    @PostMapping("/addOrUpdateAprEmp")
    public Result addOrUpdateAprEmp(AprEmpInfoVO chAprEmp) {
        if (chAprEmp == null) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (chAprEmp.getAprOrganId() == null) {//评定机构id
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(chAprEmp.getRealName())) {//评定人员姓名
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(chAprEmp.getCertifTypeId())) {//评定人员证件类型
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(chAprEmp.getCertiCode())) {//评定人员证件号码
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(chAprEmp.getGender())) {//评定人员性别
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (chAprEmp.getAge() == null) {//评定人员年龄
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(chAprEmp.getTel())) {//评定人员手机号码
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        //null == chAprEmp.getAprempTypeIdF() ||
        if (null == chAprEmp.getAprempTypeIdS()) {//评定人员资质id
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        //StringUtils.isBlank(chAprEmp.getAprempTypeNameF()) ||
        if ( StringUtils.isBlank(chAprEmp.getAprempTypeNameS())) {//评定人员资质
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (chAprEmp.getEmpType() == null) {//评定人员类型
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (chAprEmp.getEduId() == null) {//评定人员文化水平
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (chAprEmp.getGroupLeader() == null) {//评定人员是否是组长
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }

        aprOrganService.addOrUpdateAprEmp(chAprEmp);
        return Result.success();
    }

    /**
     * 更改评定机构人员状态
     *
     * @param empId  评定机构人员id
     * @param isLock 状态 有效：0；失效：1
     */
    @DeleteMapping("/updateAprEmpStatus")
    public Result updateAprEmpStatus(String empId, Integer isLock) {
        if (StringUtils.isBlank(empId) || isLock == null || (isLock != 1 && isLock != 0)) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        aprOrganService.updateAprEmpStatus(empId, isLock);
        return Result.success();
    }

    /**
     * 获取评定机构人员资质
     *
     * @param parAprempTypeId 父级鉴定人员类型ID(如果没有就查询所有父类型资质)
     */
    @GetMapping("/cfgAprEmpType")
    public Result<List<ChCfgAprempType>> cfgAprEmpType(String parAprempTypeId) {
        List<ChCfgAprempType> cfgAprEmpType = aprOrganService.cfgAprEmpType(parAprempTypeId);
        return Result.success(cfgAprEmpType);
    }

    /**
     * 获取学历信息
     */
    @GetMapping("/fndEducation")
    public Result<List<ChFndEducation>> fndEducation() {
        List<ChFndEducation> fndEducation = aprOrganService.fndEducation();
        return Result.success(fndEducation);
    }

}
