package com.hungraim.ltc.governance.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.governance.dao.BuildBedMapper;
import com.hungraim.ltc.governance.service.IBuildBedService;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.governance.ChSrvBuildBed;
import com.hungraim.ltc.pojo.vo.governance.BuildBedDetailVo;
import com.hungraim.ltc.pojo.vo.governance.BuildBedSearchReq;
import com.hungraim.ltc.pojo.vo.system.AttachUpdateVo;
import com.hungraim.ltc.util.AttachType;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author jiaguokai
 */
@Slf4j
@Service
@AllArgsConstructor
public class BuildBedServiceImpl extends ServiceImpl<BuildBedMapper, ChSrvBuildBed> implements IBuildBedService {
    private final BuildBedMapper buildBedMapper;
    private final AttachFeignService attachFeignService;

    @Override
    public Result searchOrgBed(BuildBedSearchReq req) {
        if (2 == req.getStatus()) {
            req.setStatus(null);
        }
        Page page = buildBedMapper.searchOrgBed(new Page<>(req.getPageNum(), req.getPageSize()), req);
        return Result.success(page);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveOrUpdateBuildBed(BuildBedDetailVo buildBedDetailVo) {
        ChSrvBuildBed chSrvBuildBed = new ChSrvBuildBed();
        BeanUtils.copyProperties(buildBedDetailVo, chSrvBuildBed);
        chSrvBuildBed.setStatus(0); // 状态（0已建床，1已撤床）
        chSrvBuildBed.setAccountStatus(0); // 结算状态（0，未结算；1，结算中；2，已结算）
        this.saveOrUpdate(chSrvBuildBed);
        buildBedDetailVo.setBuildBedId(chSrvBuildBed.getBuildBedId());
        this.updateAttachs(buildBedDetailVo);
    }

    @Override
    public BuildBedDetailVo getBuildBedDetail(Integer buildBedId) {
        ChSrvBuildBed chSrvBuildBed = this.getById(buildBedId);
        BuildBedDetailVo buildBedDetailVo = new BuildBedDetailVo();
        BeanUtils.copyProperties(chSrvBuildBed, buildBedDetailVo);
        try {
            Result<List<ChFndAttach>> result = attachFeignService.selectAttachs(buildBedId.longValue(), AttachType.BUILD_BED.getUploadType());
            if (ResultCode.SUCCESS.getCode().equals(result.getCode())) {
                buildBedDetailVo.setFileDataList(result.getData());
            } else {
                log.error("请求获取建床附件失败：{}", result.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("请求获取建床附件失败：{}", e.getMessage());
        }
        return buildBedDetailVo;
    }

    /**
     * 调用feign api来修改附件数据
     *
     * @param buildBedDetailVo 建床详情封装对象
     */
    private void updateAttachs(BuildBedDetailVo buildBedDetailVo) {
        // zhangch 20210525 新增附件修改逻辑
        List<ChFndAttach> fileDataList = buildBedDetailVo.getFileDataList();
        AttachUpdateVo attachUpdateVo = new AttachUpdateVo();
        attachUpdateVo.setCtrlId(buildBedDetailVo.getBuildBedId().longValue());
        attachUpdateVo.setAttachType(AttachType.BUILD_BED.getUploadType());
        attachUpdateVo.setChFndAttachs(fileDataList);
        attachFeignService.updateAttachs(attachUpdateVo);
    }


}
