package com.hungraim.ltc.governance.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.governance.dao.ChCfgAprItemClauseMapper;
import com.hungraim.ltc.governance.dao.ChCfgAprItemMapper;
import com.hungraim.ltc.governance.dao.ChCfgAprMapper;
import com.hungraim.ltc.governance.service.DisableAssessmentCriteriaService;
import com.hungraim.ltc.pojo.entity.assessment.ChCfgApr;
import com.hungraim.ltc.pojo.entity.assessment.ChCfgAprItem;
import com.hungraim.ltc.pojo.entity.assessment.ChCfgAprItemClause;
import com.hungraim.ltc.pojo.vo.governance.CfgAprVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 失能评定标准service
 *
 * @author czz
 */
@Service
public class DisableAssessmentCriteriaServiceImpl implements DisableAssessmentCriteriaService {

    @Autowired
    private ChCfgAprMapper chCfgAprMapper;
    @Autowired
    private ChCfgAprItemMapper chCfgAprItemMapper;
    @Autowired
    private ChCfgAprItemClauseMapper chCfgAprItemClauseMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addOrUpdateDisableAssessmentCriteria(Long aprId, String organId, String estimateName, String remark, Integer status, JSONArray project) {
        Date date = new Date();
        ChCfgApr chCfgApr = new ChCfgApr();
        if (aprId != null) {//编辑
            //鉴定标准配置
            chCfgApr = chCfgAprMapper.selectById(aprId);
            chCfgApr.setOrganId(organId);
            chCfgApr.setEstimateName(estimateName);
            chCfgApr.setRemark(remark);
            if (chCfgApr.getFcd() == null) {
                chCfgApr.setFcd(date);//首次操作时间
            }
            chCfgApr.setLcd(date);
            chCfgApr.setStatus(status.shortValue());
            chCfgAprMapper.updateById(chCfgApr);
            chCfgAprItemMapper.delete(new QueryWrapper<ChCfgAprItem>().eq("apr_id", chCfgApr.getAprId()));
            chCfgAprItemClauseMapper.delete(new QueryWrapper<ChCfgAprItemClause>().eq("apr_id", chCfgApr.getAprId()));
        } else {//新增
            //鉴定标准配置
            chCfgApr.setOrganId(organId);
            chCfgApr.setEstimateName(estimateName);
            chCfgApr.setRemark(remark);
            chCfgApr.setEffTime(date);
            chCfgApr.setStatus(status.shortValue());
            chCfgAprMapper.insert(chCfgApr);
        }
        //鉴定项目定义
        for (int i = 0; i < project.size(); i++) {
            ChCfgAprItem item = new ChCfgAprItem();
            JSONObject itemObject = project.getJSONObject(i);
            String projectNama = itemObject.getStr("projectNama");
            String minVal = itemObject.getStr("minVal");
            String maxVal = itemObject.getStr("maxVal");
            item.setItemName(projectNama);
            item.setMinValue(Short.parseShort(minVal));
            item.setMaxValue(Short.parseShort(maxVal));
            item.setAprId(chCfgApr.getAprId());
            chCfgAprItemMapper.insert(item);
            //鉴定项目定义明细
            JSONArray projectScoreCriteria = itemObject.getJSONArray("projectScoreCriteria");
            for (int j = 0; j < projectScoreCriteria.size(); j++) {
                ChCfgAprItemClause clause = new ChCfgAprItemClause();
                JSONObject clauseObject = projectScoreCriteria.getJSONObject(j);
                String criteria = clauseObject.getStr("criteria");
                String reference = clauseObject.getStr("reference");
                clause.setClauseDesc(criteria);
                clause.setClauseValue(Short.parseShort(reference));
                clause.setItemId(item.getItemId());
                clause.setAprId(chCfgApr.getAprId());
                chCfgAprItemClauseMapper.insert(clause);
            }
        }
    }

    @Override
    public Page<CfgAprVO> getDisableAssessmentCriteriaList(Page<CfgAprVO> page, String organIds, String aprName, String status) {
        List<String> organIdList = null;
        if (StrUtil.isNotBlank(organIds)) {
            organIdList = Arrays.asList(organIds.split(","));
        }
        return chCfgAprMapper.getDisableAssessmentCriteriaList(page, organIdList, aprName, status);
    }

    @Override
    public void stopDisableAssessmentCriteria(String aprId) {
        ChCfgApr chCfgApr = chCfgAprMapper.selectById(aprId);
        chCfgApr.setStatus((short) 0);
        chCfgAprMapper.updateById(chCfgApr);
    }
}
