package com.hungraim.ltc.governance.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.governance.dao.RemoveBedMapper;
import com.hungraim.ltc.governance.service.IBuildBedService;
import com.hungraim.ltc.governance.service.IRemoveBedService;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.governance.ChSrvBuildBed;
import com.hungraim.ltc.pojo.entity.governance.ChSrvRemoveBed;
import com.hungraim.ltc.pojo.vo.governance.RemoveBedDetailVo;
import com.hungraim.ltc.pojo.vo.system.AttachUpdateVo;
import com.hungraim.ltc.util.AttachType;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author jiaguokai
 */
@Service
@Slf4j
@AllArgsConstructor
public class RemoveBedServiceImpl extends ServiceImpl<RemoveBedMapper, ChSrvRemoveBed> implements IRemoveBedService {

    private final AttachFeignService attachFeignService;
    private final IBuildBedService buildBedService;

    @Override
    public Integer removeOne(ChSrvRemoveBed removeBed) {
        return null;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveOrUpdateRemoveBed(RemoveBedDetailVo removeBedDetailVo) {
        // 添加撤床记录
        ChSrvRemoveBed chSrvRemoveBed = new ChSrvRemoveBed();
        BeanUtils.copyProperties(removeBedDetailVo, chSrvRemoveBed);
        chSrvRemoveBed.setStatus(0); // 状态(0有效，1失效)
        this.saveOrUpdate(chSrvRemoveBed);
        removeBedDetailVo.setRemoveBedId(chSrvRemoveBed.getRemoveBedId());
        this.updateAttachs(removeBedDetailVo);

        // 变更建床记录为已撤床
        Integer buildBedId = chSrvRemoveBed.getBuildBedId();
        ChSrvBuildBed srvBuildBed = buildBedService.getById(buildBedId);
        srvBuildBed.setStatus(1); // 状态（0已建床，1已撤床）
        srvBuildBed.setAccountStatus(1); // 结算状态（0，未结算；1，结算中；2，已结算）
        buildBedService.updateById(srvBuildBed);
    }

    @Override
    public RemoveBedDetailVo getRemoveBedDetail(Integer buildBedId) {
        List<ChSrvRemoveBed> list = this.list(new LambdaQueryWrapper<ChSrvRemoveBed>().eq(ChSrvRemoveBed::getBuildBedId, buildBedId).eq(ChSrvRemoveBed::getStatus, 0));
        if (list != null && list.size() > 0) {
            ChSrvRemoveBed chSrvRemoveBed = list.get(0);
            RemoveBedDetailVo removeBedDetailVo = new RemoveBedDetailVo();
            BeanUtils.copyProperties(chSrvRemoveBed, removeBedDetailVo);
            try {
                Result<List<ChFndAttach>> result = attachFeignService.selectAttachs(chSrvRemoveBed.getRemoveBedId().longValue(), AttachType.REMOVE_BED.getUploadType());
                if (ResultCode.SUCCESS.getCode().equals(result.getCode())) {
                    removeBedDetailVo.setFileDataList(result.getData());
                } else {
                    log.error("请求获取建床附件失败：{}", result.getMsg());
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.error("请求获取建床附件失败：{}", e.getMessage());
            }
            return removeBedDetailVo;
        }
        return null;
    }

    /**
     * 调用feign api来修改附件数据
     *
     * @param removeBedDetailVo 撤床详情封装对象
     */
    private void updateAttachs(RemoveBedDetailVo removeBedDetailVo) {
        // zhangch 20210525 新增附件修改逻辑
        List<ChFndAttach> fileDataList = removeBedDetailVo.getFileDataList();
        AttachUpdateVo attachUpdateVo = new AttachUpdateVo();
        attachUpdateVo.setCtrlId(removeBedDetailVo.getRemoveBedId().longValue());
        attachUpdateVo.setAttachType(AttachType.REMOVE_BED.getUploadType());
        attachUpdateVo.setChFndAttachs(fileDataList);
        attachFeignService.updateAttachs(attachUpdateVo);
    }
}
