package com.hungraim.ltc.governance.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.governance.dao.ChSrvProgramMapper;
import com.hungraim.ltc.governance.dao.ChSrvProgramSetMapper;
import com.hungraim.ltc.governance.dao.ChSrvSendMapper;
import com.hungraim.ltc.governance.service.SrvProgramService;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvProgram;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvProgramSet;
import com.hungraim.ltc.pojo.vo.governance.ProgramDetailVo;
import com.hungraim.ltc.pojo.vo.governance.ProgramListVo;
import com.hungraim.ltc.pojo.vo.governance.ProgramSetAndProjectVo;
import com.hungraim.ltc.pojo.vo.governance.SvrSendListVo;
import com.hungraim.ltc.pojo.vo.system.AttachUpdateVo;
import com.hungraim.ltc.util.AttachType;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;


@Service
@Slf4j
public class SrvProgramServiceImpl implements SrvProgramService {

    @Autowired
    private ChSrvSendMapper chSrvSendMapper;

    @Autowired
    private ChSrvProgramMapper chSrvProgramMapper;

    @Autowired
    private ChSrvProgramSetMapper chSrvProgramSetMapper;

    @Autowired
    private AttachFeignService attachFeignService;

    @Override
    public Page<SvrSendListVo> getProgramList(Page<SvrSendListVo> page, Long srvOrganId, String realName, String realTel, String programStatus) {
        return chSrvSendMapper.selectSrvSendList(page, srvOrganId, realName, realTel, programStatus);
    }

    @Override
    public Page<ProgramListVo> getProgramSetList(Page<ProgramListVo> page, Long disabInfoId, Long srvOrganId, String programStatus) {
        return chSrvProgramMapper.getProgramList(page, disabInfoId, srvOrganId, programStatus);
    }

    @Override
    public List<ChSrvProgram> getProgramSetCountByProgStatus(Long disabInfoId, Long srvOrganId, String programStatus) {
        QueryWrapper<ChSrvProgram> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("DISAB_INFO_ID", disabInfoId).eq("SRV_ORGAN_ID", srvOrganId).ne("PROG_STATUS", programStatus);
        List<ChSrvProgram> chSrvPrograms = chSrvProgramMapper.selectList(queryWrapper);
        return chSrvPrograms;
    }

    @Override
    public ProgramDetailVo getProgramDetail(Long programId) {
        ProgramDetailVo programDetail = chSrvProgramMapper.getProgramDetail(programId);
        // 返回对应的附件列表
        Result<List<ChFndAttach>> listResult = attachFeignService.selectAttachs(programId, AttachType.PROGRAM.getUploadType());
        if (!listResult.getCode().equals(ResultCode.SUCCESS.getCode())) {
            log.info("请求获取附件列表错误：错误码{},错误信息:{}", listResult.getCode(), listResult.getMsg());
        } else {
            List<ChFndAttach> data = listResult.getData();
            programDetail.setFileDataList(data);
        }
        return programDetail;
    }

    @Override
    public ProgramDetailVo getProgramDetailWithAdd(Long disabInfoId, Long srvOrganId) {
        return chSrvProgramMapper.getProgramDetailWithAdd(disabInfoId, srvOrganId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addProgramAndProgramSetList(ProgramDetailVo programDetailVo) {
        Date date = new Date();
        Long programId = programDetailVo.getProgramId();
        if (null == programId) {
            // 添加服务方案操作
            // 1.添加服务方案记录
            ChSrvProgram chSrvProgram = new ChSrvProgram();
            chSrvProgram.setSrvOrganId(programDetailVo.getSrvOrganId());
            chSrvProgram.setDisabInfoId(programDetailVo.getDisabInfoId());
            chSrvProgram.setSrvModeId(programDetailVo.getSrvModeId());
            chSrvProgram.setProgramDate(programDetailVo.getProgramDate());
            chSrvProgram.setProgramEdnDate(programDetailVo.getProgramEndDate());
            chSrvProgram.setProgStatus(0);
            chSrvProgram.setChargeType(0L);
            chSrvProgram.setEffTime(date);
            chSrvProgramMapper.insert(chSrvProgram);
            programId = chSrvProgram.getProgramId();
            // 2.使用服务方案记录的id来添加服务方案项目记录
            Long newProgramId = chSrvProgram.getProgramId();
            List<ProgramSetAndProjectVo> programSetAndProjectVos = programDetailVo.getProgramSetAndProjectVos();
            for (ProgramSetAndProjectVo programSetAndProjectVo : programSetAndProjectVos) {
                Short isConf = programSetAndProjectVo.getIsConf();
                if (isConf.intValue() == 0) {
                    // 没有配置这条服务方案，直接略过这条记录，无需添加
                    continue;
                }
                this.addProgramSet(programDetailVo, date, newProgramId, programSetAndProjectVo);
            }
            this.updateAttachs(programDetailVo.getFileDataList(), chSrvProgram.getProgramId());
        } else {
            // 修改操作,修改操作就是对服务方案项目进行修改，两种情况
            List<ProgramSetAndProjectVo> programSetAndProjectVos = programDetailVo.getProgramSetAndProjectVos();
            for (ProgramSetAndProjectVo programSetAndProjectVo : programSetAndProjectVos) {
                Short isConf = programSetAndProjectVo.getIsConf();
                if (programSetAndProjectVo.getSetId() != null) {
                    if (isConf.intValue() == 0) {
                        // 没有配置这条服务方案，直接删除对应的记录
                        chSrvProgramSetMapper.deleteById(programSetAndProjectVo.getSetId());
                    }
                    // 修改原有的服务方案项目
                    ChSrvProgramSet chSrvProgramSet = new ChSrvProgramSet();
                    chSrvProgramSet.setSetId(programSetAndProjectVo.getSetId());
                    chSrvProgramSet.setPlanningFrequency(programSetAndProjectVo.getPlanningFrequency());
                    chSrvProgramSet.setPlanningFrequencyType(programSetAndProjectVo.getPlanningFrequencyType());
                    chSrvProgramSet.setRemark(programSetAndProjectVo.getRemark());
                    chSrvProgramSetMapper.updateById(chSrvProgramSet);
                } else {
                    if (isConf.intValue() == 0) {
                        // 没有配置这条服务方案，直接略过这条记录，无需添加
                        continue;
                    }
                    // 新增一条新的服务方案项目
                    this.addProgramSet(programDetailVo, date, programId, programSetAndProjectVo);
                }
            }
            this.updateAttachs(programDetailVo.getFileDataList(), programId);
        }

        //修改附件
        this.updateAttachs(programDetailVo.getFileDataList(), programId);
    }

    @Override
    public void endProgram(Long programId, String endReason) {
        ChSrvProgram chSrvProgram = new ChSrvProgram();
        chSrvProgram.setProgramId(programId);
        chSrvProgram.setEndReason(endReason);
        chSrvProgram.setProgStatus(2);
        chSrvProgramMapper.updateById(chSrvProgram);
    }

    @Override
    public Result<List<ChFndAttach>> programFileList(Long programId) {
        Result<List<ChFndAttach>> listResult = null;
        try {
            // 返回对应的附件列表
            listResult = attachFeignService.selectAttachs(programId, AttachType.PROGRAM.getUploadType());
            if (!listResult.getCode().equals(ResultCode.SUCCESS.getCode())) {
                log.info("请求获取附件列表错误：错误码{},错误信息:{}", listResult.getCode(), listResult.getMsg());
            }
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
        }
        return listResult;
    }

    private void addProgramSet(ProgramDetailVo programDetailVo, Date date, Long newProgramId, ProgramSetAndProjectVo programSetAndProjectVo) {
        ChSrvProgramSet chSrvProgramSet = new ChSrvProgramSet();
        chSrvProgramSet.setItemId(programSetAndProjectVo.getProjectId());
        chSrvProgramSet.setIsConf(programSetAndProjectVo.getIsConf());
        chSrvProgramSet.setItemTimes(programSetAndProjectVo.getItemTimes());
        chSrvProgramSet.setEffTime(date);
        chSrvProgramSet.setProgramId(newProgramId);
        chSrvProgramSet.setSrvModeId(programDetailVo.getSrvModeId());
        chSrvProgramSet.setItemType(programSetAndProjectVo.getCategoryType());
        chSrvProgramSet.setChooseFlag(programSetAndProjectVo.getIsNeed());
        chSrvProgramSet.setPlanningFrequency(programSetAndProjectVo.getPlanningFrequency());
        chSrvProgramSet.setPlanningFrequencyType(programSetAndProjectVo.getPlanningFrequencyType());
        chSrvProgramSet.setRemark(programSetAndProjectVo.getRemark());
        chSrvProgramSetMapper.insert(chSrvProgramSet);
    }


    /**
     * 调用feign api来修改附件数据
     */
    private void updateAttachs(List<ChFndAttach> fileDataList, Long ctrlId) {
        // 新增附件修改逻辑
        AttachUpdateVo attachUpdateVo = new AttachUpdateVo();
        attachUpdateVo.setCtrlId(ctrlId);
        attachUpdateVo.setAttachType(AttachType.PROGRAM.getUploadType());
        attachUpdateVo.setChFndAttachs(fileDataList);
        attachFeignService.updateAttachs(attachUpdateVo);
    }
}
