package com.hungraim.ltc.api;

import com.hungraim.ltc.pojo.entity.system.SystemUser;
import com.hungraim.ltc.util.Result;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.List;

/**
 * @author hubin
 */
@FeignClient("LTC-SYSTEM-ADMIN-SERVICE")
public interface SystemAdminFeignService {

    /**
     * feign api 根据用户账号获取用户基本信息
     *
     * @param account 用户账号
     * @return 用户信息
     */
    @GetMapping("/api.system/user/loadUserInfoByAccount/{account}")
    Result<SystemUser> loadUserInfoByAccount(@PathVariable("account") String account);

    /**
     * feign api 根据用户id获取用户权限列表
     *
     * @param id 用户id
     * @return 权限列表
     */
    @GetMapping("/api.system/user/loadUserRolesByUserId/{id}")
    Result<List<Long>> loadUserRolesByUserId(@PathVariable("id") Long id);


  /**
   * feign api 根据用户表关联的marketerCode获取用户绑定的营销员信息
   * @param code marketerCode
   * @return 营销员信息
   */
//  @GetMapping("/api.system/user/loadUserDetailByCode/{code}")
//  Result<SystemMarketer> loadUserDetailByCode(@PathVariable String code);
}
